<?php
/**
 * @version $Id: dbquery.php 1487 2012-04-17 08:27:46Z roosit $
 * @package Abricos
 * @subpackage Company
 * @copyright Copyright (C) 2008 Abricos. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin (roosit@abricos.org)
 */

/**
 * Статичные функции запросов к базе данных
 * 
 * @package Abricos
 * @subpackage Company
 */
class CompanyQuery {
	
	public static function Days(Ab_Database $db, $userid, $data){
		$sql = CalendarQuery::DaysSQL($db, $userid, $data);
		
		$sql .= " AND permlevel >= 1";
		
		return $db->query_read($sql);
	}

	public static function CalendarOptionsAppend(Ab_Database $db, $userid, $d){
		if ($d->cfg == 'act'){
			$sql = "
				INSERT INTO ".$db->prefix."cmpn_calperm 
					(userid, options) VALUES (
					".bkint($userid).",
					'".bkstr($d->ops)."'
				)
			";
		}else{
			$sql = "
				INSERT INTO ".$db->prefix."cmpn_calperm 
					(userid, calin, calout) VALUES (
					".bkint($userid).",
					'".bkstr($d->cin)."',
					'".bkstr($d->cout)."'
				)
			";
		}
		$db->query_write($sql);
	}
	
	public static function CalendarOptionsUpdate(Ab_Database $db, $userid, $d){
		$set = "";
		if ($d->act == 'cfg'){
			$set = "
				options='".bkstr($d->ops)."'
			";
		}else{
			$set = "
				calin='".bkstr($d->cin)."',
				calout='".bkstr($d->cout)."'
			";
		}
		$sql = "
			UPDATE ".$db->prefix."cmpn_calperm 
			SET ".$set."
			WHERE userid=".bkint($userid)."
		";
		$db->query_write($sql);
	}
	
	public static function CalendarPermission(Ab_Database $db, $userid, $firstRow = false){
		$info = CompanyQuery::Employee($db, $userid, true);
		$sql = "
			(SELECT 
				e.userid as id,
				c.calpermid as cpid,
				c.calin as cin,
				c.calout as cout,
				c.options as ops
			FROM ".$db->prefix."user e
			LEFT JOIN ".$db->prefix."cmpn_calperm c ON e.userid = c.userid
			WHERE e.userid=".bkint($userid).")
			UNION
			(SELECT 
				e.userid as id,
				'' as cpid,
				'' as cin,
				'' as cout,
				'' as ops
			FROM ".$db->prefix."user e
			LEFT JOIN ".$db->prefix."cmpn_calperm c ON e.userid = c.userid
			WHERE c.calout REGEXP '#".$info['id']."[^0-9]?')
		";
		if ($firstRow){
			return $db->query_first($sql);
		}
		return $db->query_read($sql);
	}
	
	public static function CalendarCheckAccess(Ab_Database $db, $userid, $getAccessEmpId){
		$sql = "
			SELECT
				e.userid as id
			FROM ".$db->prefix."user e
			LEFT JOIN ".$db->prefix."cmpn_calperm c ON e.userid = c.userid
			WHERE e.userid=".bkint($userid)." AND c.calout REGEXP '#".$getAccessEmpId."[^0-9]?'
		";
		$row = $db->query_first($sql);
		return !empty($row);
	}
	
	public static function EmployeeUpdate(Ab_Database $db, $d){
		$sql = "
			UPDATE ".$db->prefix."user
			SET
				lastname='".bkstr($d->elnm)."',
				firstname='".bkstr($d->efnm)."',
				patronymic='".bkstr($d->epnc)."',
				postid=".bkint($d->postid).",
				deptid=".bkint($d->deptid).",
				room='".bkstr($d->rm)."',
				phones='".bkstr($d->phs)."'
			WHERE userid=".bkint($d->id)."
		";
		$db->query_write($sql);
	}

	public static function Employee(Ab_Database $db, $userid, $retarray = false){
		$sql = "
			SELECT 
				u.userid as id,
				u.username as unm,
				u.lastname as elnm,
				u.firstname as efnm,
				u.patronymic as epnc,
				u.postid as postid,
				u.deptid as deptid,
				u.room as rm,
				u.phones as phs,
				p.level as lvl,
				p.level IS NULL as nop,
				!(ug.groupid IS NULL) as isemp
			FROM ".$db->prefix."user u
			LEFT JOIN ".$db->prefix."usergroup ug ON ug.userid = u.userid
			LEFT JOIN ".$db->prefix."group g ON g.groupid = ug.groupid
			LEFT JOIN ".$db->prefix."cmpn_post p ON u.postid = p.postid
			WHERE u.userid=".bkint($userid)." AND g.groupkey='".CompanyGroup::EMPLOYEE."'
			LIMIT 1
		";
		return $retarray ? $db->query_first($sql) : $db->query_read($sql);
	}
	
	public static function EmployeeList(Ab_Database $db){
		$sql = "
			SELECT 
				u.userid as id,
				u.username as unm,
				u.lastname as elnm,
				u.firstname as efnm,
				u.patronymic as epnc,
				u.room as rm,
				u.phones as phs,
				u.postid as ptid,
				u.deptid as dtid,
				p.level as lvl,
				p.level IS NULL as nop,
				IF(d.ord IS NULL, 99, d.ord) as ord
			FROM ".$db->prefix."usergroup ug
			LEFT JOIN ".$db->prefix."group g ON g.groupid = ug.groupid
			LEFT JOIN ".$db->prefix."user u ON ug.userid = u.userid
			LEFT JOIN ".$db->prefix."cmpn_post p ON u.postid = p.postid
			LEFT JOIN ".$db->prefix."cmpn_dept d ON u.deptid = d.deptid
			WHERE g.groupkey='".CompanyGroup::EMPLOYEE."'
			ORDER BY ord, dtid, nop, lvl, elnm, efnm
		";
		return $db->query_read($sql);
	}
	
	public static function PostUpdate(Ab_Database $db, $d){
		$sql = "
			UPDATE ".$db->prefix."cmpn_post
			SET 
				name='".bkstr($d->nm)."',
				level=".bkint($d->lvl)."
			WHERE postid=".bkint($d->id)."
		";
		$db->query_write($sql);
	}
	
	public static function PostAppend(Ab_Database $db, $d){
		$sql = "
			INSERT INTO ".$db->prefix."cmpn_post (name, level) VALUES (
				'".bkstr($d->nm)."',
				".bkint($d->lvl)."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function PostList(Ab_Database $db){
		$sql = "
			SELECT 
				postid as id,
				name as nm,
				level as lvl
			FROM ".$db->prefix."cmpn_post
			ORDER BY lvl
		";
		return $db->query_read($sql);
	}

	public static function DeptUpdate(Ab_Database $db, $d){
		$sql = "
			UPDATE ".$db->prefix."cmpn_dept
			SET 
				name='".bkstr($d->nm)."',
				ord=".bkint($d->ord)."
			WHERE deptid=".bkint($d->id)."
		";
		$db->query_write($sql);
	}
	
	public static function DeptAppend(Ab_Database $db, $d){
		$sql = "
			INSERT INTO ".$db->prefix."cmpn_dept (name, ord) VALUES (
				'".bkstr($d->nm)."',
				".bkint($d->ord)."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function DeptList(Ab_Database $db){
		$sql = "
			SELECT 
				deptid as id,
				name as nm,
				ord
			FROM ".$db->prefix."cmpn_dept
			ORDER BY ord, nm
		";
		return $db->query_read($sql);
	}

}


?>